import unittest

import src.upload as upload

# This unit test uses Python's built-in unit testing framework
# See https://docs.python.org/3/library/unittest.html for more information


class UploadTest(unittest.TestCase):
    def test_correctlyDetectsLargerVersionNumber(self):
        self.assertTrue(upload.versionNumberGreater("1.0.12", "1.0.3"))
        self.assertTrue(upload.versionNumberGreater("1.1.2", "1.0.5"))
        self.assertTrue(upload.versionNumberGreater("0.1.2", "0.0.9"))
        self.assertFalse(upload.versionNumberGreater("0.0.2", "1.0.0"))
        self.assertFalse(upload.versionNumberGreater("5.4.3", "5.5.1"))
        self.assertFalse(upload.versionNumberGreater("1.0.2", "1.0.2"))

    def test_getLargestVersionNumber(self):
        versions = ["1.0.12", "1.0.3", "1.1.2",
                    "1.0.5", "5.4.3", "5.5.1", "0.0.2", "1.0.0"]
        largest = upload.getLargestVersionNumber(versions)
        self.assertEqual(largest, "5.5.1")

    def test_isValidVersionNumber(self):
        self.assertTrue(upload.isValidVersionNumber("0.0.1"))
        self.assertTrue(upload.isValidVersionNumber("0.12.519"))
        self.assertTrue(upload.isValidVersionNumber("9.0.1"))
        self.assertFalse(upload.isValidVersionNumber("1.0"))
        self.assertFalse(upload.isValidVersionNumber("0.4.g"))
        self.assertFalse(upload.isValidVersionNumber("test"))


if __name__ == "__main__":
    unittest.main()



import tkinter as tk

from src import strings
from src.ui.DialogWindow import ActionDialogWindow, InfoDialogWindow


class AuthorInfoSelector:
    def __init__(self):
        self.window = tk.Tk()
        self.window.title("PyPI Maker")
        centerX = (self.window.winfo_screenwidth() - 650) // 2
        centerY = (self.window.winfo_screenheight() - 310) // 2
        self.window.geometry(f"650x310+{centerX}+{centerY}")

        self.names = []
        self.emails = []
        self.correspondingEmail = ""

        labelPrimary = tk.Label(self.window, text="Corresponding Author")

        authorFrame1 = tk.Frame(self.window)
        labelName1 = tk.Label(self.window, text="Name:")
        self.entryName1 = tk.Entry(self.window, width=25)
        labelEmail1 = tk.Label(self.window, text="Email:")
        self.entryEmail1 = tk.Entry(self.window, width=25)
        labelName1.pack(in_=authorFrame1, side=tk.LEFT)
        self.entryName1.pack(in_=authorFrame1, side=tk.LEFT)
        labelEmail1.pack(in_=authorFrame1, side=tk.LEFT)
        self.entryEmail1.pack(in_=authorFrame1, side=tk.LEFT)

        labelOthers = tk.Label(self.window, text="Other Authors")

        authorFrame2 = tk.Frame(self.window)
        labelName2 = tk.Label(self.window, text="Name:")
        self.entryName2 = tk.Entry(self.window, width=25)
        labelEmail2 = tk.Label(self.window, text="Email:")
        self.entryEmail2 = tk.Entry(self.window, width=25)
        labelName2.pack(in_=authorFrame2, side=tk.LEFT)
        self.entryName2.pack(in_=authorFrame2, side=tk.LEFT)
        labelEmail2.pack(in_=authorFrame2, side=tk.LEFT)
        self.entryEmail2.pack(in_=authorFrame2, side=tk.LEFT)

        authorFrame3 = tk.Frame(self.window)
        labelName3 = tk.Label(self.window, text="Name:")
        self.entryName3 = tk.Entry(self.window, width=25)
        labelEmail3 = tk.Label(self.window, text="Email:")
        self.entryEmail3 = tk.Entry(self.window, width=25)
        labelName3.pack(in_=authorFrame3, side=tk.LEFT)
        self.entryName3.pack(in_=authorFrame3, side=tk.LEFT)
        labelEmail3.pack(in_=authorFrame3, side=tk.LEFT)
        self.entryEmail3.pack(in_=authorFrame3, side=tk.LEFT)

        authorFrame4 = tk.Frame(self.window)
        labelName4 = tk.Label(self.window, text="Name:")
        self.entryName4 = tk.Entry(self.window, width=25)
        labelEmail4 = tk.Label(self.window, text="Email:")
        self.entryEmail4 = tk.Entry(self.window, width=25)
        labelName4.pack(in_=authorFrame4, side=tk.LEFT)
        self.entryName4.pack(in_=authorFrame4, side=tk.LEFT)
        labelEmail4.pack(in_=authorFrame4, side=tk.LEFT)
        self.entryEmail4.pack(in_=authorFrame4, side=tk.LEFT)

        authorFrame5 = tk.Frame(self.window)
        labelName5 = tk.Label(self.window, text="Name:")
        self.entryName5 = tk.Entry(self.window, width=25)
        labelEmail5 = tk.Label(self.window, text="Email:")
        self.entryEmail5 = tk.Entry(self.window, width=25)
        labelName5.pack(in_=authorFrame5, side=tk.LEFT)
        self.entryName5.pack(in_=authorFrame5, side=tk.LEFT)
        labelEmail5.pack(in_=authorFrame5, side=tk.LEFT)
        self.entryEmail5.pack(in_=authorFrame5, side=tk.LEFT)

        authorFrame6 = tk.Frame(self.window)
        labelName6 = tk.Label(self.window, text="Name:")
        self.entryName6 = tk.Entry(self.window, width=25)
        labelEmail6 = tk.Label(self.window, text="Email:")
        self.entryEmail6 = tk.Entry(self.window, width=25)
        labelName6.pack(in_=authorFrame6, side=tk.LEFT)
        self.entryName6.pack(in_=authorFrame6, side=tk.LEFT)
        labelEmail6.pack(in_=authorFrame6, side=tk.LEFT)
        self.entryEmail6.pack(in_=authorFrame6, side=tk.LEFT)

        authorFrame7 = tk.Frame(self.window)
        labelName7 = tk.Label(self.window, text="Name:")
        self.entryName7 = tk.Entry(self.window, width=25)
        labelEmail7 = tk.Label(self.window, text="Email:")
        self.entryEmail7 = tk.Entry(self.window, width=25)
        labelName7.pack(in_=authorFrame7, side=tk.LEFT)
        self.entryName7.pack(in_=authorFrame7, side=tk.LEFT)
        labelEmail7.pack(in_=authorFrame7, side=tk.LEFT)
        self.entryEmail7.pack(in_=authorFrame7, side=tk.LEFT)

        authorFrame8 = tk.Frame(self.window)
        labelName8 = tk.Label(self.window, text="Name:")
        self.entryName8 = tk.Entry(self.window, width=25)
        labelEmail8 = tk.Label(self.window, text="Email:")
        self.entryEmail8 = tk.Entry(self.window, width=25)
        labelName8.pack(in_=authorFrame8, side=tk.LEFT)
        self.entryName8.pack(in_=authorFrame8, side=tk.LEFT)
        labelEmail8.pack(in_=authorFrame8, side=tk.LEFT)
        self.entryEmail8.pack(in_=authorFrame8, side=tk.LEFT)

        self.buttonDone = tk.Button(
            self.window, text="Done", command=self.checkDone, fg="blue")

        labelPrimary.pack()
        authorFrame1.pack()
        labelOthers.pack()
        authorFrame2.pack()
        authorFrame3.pack()
        authorFrame4.pack()
        authorFrame5.pack()
        authorFrame6.pack()
        authorFrame7.pack()
        authorFrame8.pack()
        self.buttonDone.pack()

        self.window.protocol("WM_DELETE_WINDOW", quit)
        self.window.bind("<Return>", self.checkDoneWithReturnKey)

        self.window.mainloop()

    def checkDone(self):
        names = [self.entryName1.get(), self.entryName2.get(), self.entryName3.get(),
                 self.entryName4.get(), self.entryName5.get(), self.entryName6.get(),
                 self.entryName7.get(), self.entryName8.get()]
        emails = [self.entryEmail1.get(), self.entryEmail2.get(), self.entryEmail3.get(),
                  self.entryEmail4.get(), self.entryEmail5.get(), self.entryEmail6.get(),
                  self.entryEmail7.get(), self.entryEmail8.get()]

        noEmailsEntered = True
        for name, email in zip(names, emails):
            if email != "":
                noEmailsEntered = False
                if name == "":
                    InfoDialogWindow(
                        "Invalid entry", strings.MISSING_AUTHOR_NAMES_TEXT)
                    return

        if names[0] == "":
            ActionDialogWindow("Invalid entry", strings.MISSING_FIRST_NAME_TEXT,
                               positiveAction=self.doneWithNoAuthors)
            return

        if emails[0] == "" and not noEmailsEntered:
            ActionDialogWindow(
                "Invalid entry", strings.MISSING_FIRST_AUTHOR_EMAIL_TEXT)

        self.done(names, emails)

    def checkDoneWithReturnKey(self, _):
        self.checkDone()

    def done(self, names, emails):
        self.names, self.emails = self.getFilteredNamesAndEmails(names, emails)
        self.correspondingEmail = self.getCorrespondingEmail()
        self.window.destroy()
        self.window.quit()

    def doneWithNoAuthors(self):
        self.names = []
        self.emails = []
        self.window.destroy()
        self.window.quit()

    def getCorrespondingEmail(self):
        for email in self.emails:
            if email != "":
                return email

    def getFilteredNamesAndEmails(self, names, emails):
        filteredNames = []
        filteredEmails = []
        for index, name in enumerate(names):
            if name != "":
                filteredNames.append(names[index])
                filteredEmails.append(emails[index])
        return filteredNames, filteredEmails



import unittest

import src.files as files

# This unit test uses Python's built-in unit testing framework
# See https://docs.python.org/3/library/unittest.html for more information


class FilesTest(unittest.TestCase):
    def test_getNamesAndEmailsVerticallySpaced_spacesInfoProperly(self):
        authorsArray = ["John Doe", "Jane Toe"]
        emailsArray = ["john@gmail.com", "jane@hotmail.com"]
        expectedOutput = "John Doe (john@gmail.com)\n\nJane Toe (jane@hotmail.com)"

        output = files.getNamesAndEmailsVerticallySpaced(
            authorsArray, emailsArray)

        self.assertEqual(output, expectedOutput)

    def test_getAuthorsNameLine(self):
        emptyNameLine = files.getAuthorsNameLine([])
        singleNameLine = files.getAuthorsNameLine(["John Doe"])
        doubleNameLine = files.getAuthorsNameLine(["John Doe", "Jane Toe"])
        tripleNameLine = files.getAuthorsNameLine(
            ["John Doe", "Jane Toe", "Bob Smith"])

        self.assertEqual(emptyNameLine, "")
        self.assertEqual(singleNameLine, "\n    author='John Doe',")
        self.assertEqual(
            doubleNameLine, "\n    author='John Doe and Jane Toe',")
        self.assertEqual(
            tripleNameLine, "\n    author='John Doe, Jane Toe, and Bob Smith',")

    def test_getAuthorNamesInline(self):
        noNamesLine = files.getAuthorNamesInline([])
        oneNameLine = files.getAuthorNamesInline(["John Doe"])
        twoNamesLine = files.getAuthorNamesInline(["John Doe", "Jane Toe"])
        threeNamesLine = files.getAuthorNamesInline(
            ["John Doe", "Jane Toe", "Bob Smith"])

        self.assertEqual(noNamesLine, "")
        self.assertEqual(oneNameLine, "John Doe")
        self.assertEqual(twoNamesLine, "John Doe and Jane Toe")
        self.assertEqual(threeNamesLine, "John Doe, Jane Toe, and Bob Smith")

    def test_getAuthorEmailLine(self):
        noEmailOutput = files.getAuthorEmailLine("")
        emailOutput = files.getAuthorEmailLine("john@gmail.com")

        self.assertEqual(noEmailOutput, "")
        self.assertEqual(emailOutput, "\n    author_email='john@gmail.com',")

    def test_getDescriptionLine(self):
        noDescriptionOutput = files.getDescriptionLine("")
        descriptionOutput = files.getDescriptionLine("fake description")

        self.assertEqual(noDescriptionOutput, "")
        self.assertEqual(descriptionOutput,
                         "\n    description='fake description',")

    def test_getGithubInfo_returnsListOfEmptyStrings_whenUsernameIsEmptyString(self):
        self.assertEqual(files.getGithubInfo("", "testProjectName"), ["", ""])

    def test_getClassifiers_returnsEmptyString_whenClassifierIsDefaultOrNotApplicable(self):
        self.assertEqual(files.getClassifiers("N/A"), "")
        self.assertEqual(files.getClassifiers("Select..."), "")


if __name__ == "__main__":
    unittest.main()



import tkinter as tk
from tkinter import filedialog

from src import strings
import src.ui.navigation as navigation
from src.ui.AuthorInfoSelector import AuthorInfoSelector
from src.ui.DialogWindow import ActionDialogWindow, DialogWindow, InfoDialogWindow

EMPTY_DIRECTORY_SET = set(["", " ", "/"])


class OptionSelector:
    def __init__(self):
        self.window = tk.Tk()
        self.window.title("PyPI Maker")
        centerX = (self.window.winfo_screenwidth() - 650) // 2
        centerY = (self.window.winfo_screenheight() - 250) // 2
        self.window.geometry(f"650x250+{centerX}+{centerY}")

        self.filepath = ""
        self.projectName = ""
        self.authorsArray = []
        self.emailsArray = []
        self.correspondingEmail = ""
        self.githubUsername = ""
        self.shortDescription = ""
        self.intendedAudience = ""
        self.intendedAudiences = strings.INTENDED_AUDIENCES_LIST
        self.classifiers = False
        self.mainFunctionPath = strings.DEFAULT_MAIN_FUNCTION_PATH

        filepathFrame = tk.Frame(self.window)
        self.labelSelectedPath = tk.Label(
            self.window, text=strings.SELECT_FOLDER_TEXT, fg="red", bg="white")
        buttonSelectFolder = tk.Button(
            self.window, text="Browse...", command=self.browseFolders)
        self.showFilepath()
        self.labelSelectedPath.pack(in_=filepathFrame, side=tk.LEFT)
        buttonSelectFolder.pack(in_=filepathFrame, side=tk.LEFT)

        navigation = tk.Frame(self.window)
        labelIntendedAudienceSelect = tk.Label(
            self.window, text="Intended Audience:")
        self.menuIntendedAudienceSelectVar = tk.StringVar(self.window)
        self.menuIntendedAudienceSelectVar.set("Select...")
        menuIntendedAudienceSelect = tk.OptionMenu(
            self.window, self.menuIntendedAudienceSelectVar, *self.intendedAudiences)
        labelIntendedAudienceSelect.pack(in_=navigation, side=tk.LEFT)
        menuIntendedAudienceSelect.pack(in_=navigation, side=tk.LEFT)

        frameProjectName = tk.Frame(self.window)
        labelProjectName = tk.Label(self.window, text="Project Name:")
        self.entryProjectName = tk.Entry(self.window, width=25)
        labelProjectName.pack(in_=frameProjectName, side=tk.LEFT)
        self.entryProjectName.pack(in_=frameProjectName, side=tk.LEFT)

        labelProjectNameDescription = tk.Label(
            self.window,
            text=strings.PROJECT_NAME_DESCRIPTION_TEXT,
            font=("Helvetica 9", 10)
        )

        frameGithubUsername = tk.Frame(self.window)
        labelGithubUsername = tk.Label(
            self.window, text="Project GitHub Username:")
        self.entryGithubUsername = tk.Entry(self.window, width=25)
        labelGithubUsername.pack(in_=frameGithubUsername, side=tk.LEFT)
        self.entryGithubUsername.pack(in_=frameGithubUsername, side=tk.LEFT)

        labelProjectDescription = tk.Label(
            self.window, text=strings.PROJECT_DESCRIPTION_TEXT)
        self.entryProjectDescription = tk.Entry(self.window, width=50)

        self.includeAuthorNames = tk.BooleanVar(value=True)
        checkboxIncludeAuthorNames = tk.Checkbutton(
            self.window, text="Include author name(s)?", variable=self.includeAuthorNames,
            onvalue=True, offvalue=False)

        buttonDone = tk.Button(self.window, text="Done",
                               command=self.checkDone, fg="blue")

        filepathFrame.pack()
        navigation.pack()
        frameProjectName.pack()
        labelProjectNameDescription.pack()
        frameGithubUsername.pack()
        labelProjectDescription.pack()
        self.entryProjectDescription.pack()
        checkboxIncludeAuthorNames.pack()
        buttonDone.pack()

        self.window.protocol("WM_DELETE_WINDOW", quit)
        self.window.bind("<Return>", self.checkDoneWithReturnKey)

        self.window.mainloop()

    def browseFolders(self):
        chosenFolder = filedialog.askdirectory(
            initialdir=self.filepath,
            title=strings.SELECT_FOLDER_TEXT
        )
        if chosenFolder not in EMPTY_DIRECTORY_SET:
            if not navigation.hasPyFiles(chosenFolder):
                InfoDialogWindow(
                    "No Python files found",
                    strings.NO_PYTHON_FILES_IN_PROJECT_FOLDER_TEXT
                )
                return
            self.filepath = chosenFolder
        else:
            self.filepath = ""
        self.showFilepath()
        self.window.focus_force()

    def checkDone(self):
        if self.filepath in EMPTY_DIRECTORY_SET:
            InfoDialogWindow(
                "Missing info", strings.PROJECT_FOLDER_NOT_CHOSEN_TEXT)
            return

        if not navigation.hasMainFunctionInSrc(self.filepath):
            DialogWindow(
                strings.NO_MAIN_FUNCTION_TITLE,
                strings.MUST_SELECT_MAIN_FILE,
                positiveButtonText="Choose",
                negativeButtonText="Quit",
                negativeButtonAction=quit,
                negativeButtonColor="red"
            )
            self.mainFunctionPath = self.getMainFunctionPath()
            if self.mainFunctionPath == strings.DEFAULT_MAIN_FUNCTION_PATH:
                return

        self.projectName = self.entryProjectName.get()
        if self.projectName == "":
            ActionDialogWindow(
                "Missing info",
                strings.PROJECT_NAME_NOT_CHOSEN_TEXT,
                positiveAction=self.finishWithFolderNameAsProjectName
            )
            return

        chosenFolder = self.filepath.split("/")[-1]
        if chosenFolder != self.projectName:
            ActionDialogWindow(
                "Check your selection",
                strings.MISMATCHED_SELECTION_TEXT,
                positiveAction=self.done
            )
            return

        self.done()

    def checkDoneWithReturnKey(self, event):
        self.checkDone()

    def closeWindow(self):
        self.window.destroy()
        self.window.quit()

    def done(self):
        self.githubUsername = self.entryGithubUsername.get()
        self.shortDescription = self.entryProjectDescription.get()
        self.classifier = self.menuIntendedAudienceSelectVar.get()
        self.classifiers = self.classifier not in ["N/A", "Select..."]

        if self.includeAuthorNames.get():
            self.getNamesAndEmails()
        else:
            self.closeWindow()

    def finishWithFolderNameAsProjectName(self):
        chosenFolder = self.filepath.split("/")[-1]
        self.projectName = chosenFolder
        self.done()

    def getMainFunctionPath(self):
        chosenFile = filedialog.askopenfilename(
            initialdir=self.filepath,
            title='Select the file containing your "main" function'
        )

        if not navigation.mainFunctionIn(chosenFile):
            InfoDialogWindow(
                strings.NO_MAIN_FUNCTION_TITLE,
                strings.NO_MAIN_FOUND_AFTER_SELECTION_TEXT
            )
            return strings.DEFAULT_MAIN_FUNCTION_PATH

        else:
            mainFunctionPath = navigation.getPathFromBase(
                base=self.filepath, target=chosenFile)
            if mainFunctionPath == strings.DEFAULT_MAIN_FUNCTION_PATH:
                InfoDialogWindow(
                    strings.NO_MAIN_FUNCTION_TITLE,
                    strings.NO_MAIN_FOUND_AFTER_SELECTION_TEXT
                )
            return mainFunctionPath

    def getNamesAndEmails(self):
        self.closeWindow()
        authorInfoSelector = AuthorInfoSelector()
        self.authorsArray = authorInfoSelector.names
        self.emailsArray = authorInfoSelector.emails
        self.correspondingEmail = authorInfoSelector.correspondingEmail

    def showFilepath(self):
        if self.filepath in EMPTY_DIRECTORY_SET:
            self.labelSelectedPath.configure(
                text=strings.SELECT_FOLDER_TEXT, fg="red", bg="white")
        else:
            displayedFilepath = self.filepath
            if len(displayedFilepath) > 50:
                displayedFilepath = "..." + displayedFilepath[-50:]
            self.labelSelectedPath.configure(
                text=displayedFilepath, bg="white", fg="black")



import argparse
import sys

from src import strings
import src.files as files
import src.reset as reset
import src.upload as upload
from src.ui import navigation
from src.ui.OptionSelector import OptionSelector


def main(argv=sys.argv):
    parser = argparse.ArgumentParser(
        description="Example input: pypimaker generate")
    parser.add_argument(
        "command",
        type=str,
        help=strings.ARGS_DESCRIPTION_TEXT,
        nargs=1
    )
    userOption = "help" if len(sys.argv) == 1 else sys.argv[1]

    if userOption in ["generate", "-g"]:
        options = OptionSelector()
        files.generate(
            options.filepath,
            options.projectName,
            options.authorsArray,
            options.emailsArray,
            options.correspondingEmail,
            options.githubUsername,
            options.shortDescription,
            options.classifier,
            options.mainFunctionPath
        )

    elif userOption in ["upload", "-u"]:
        filepath = navigation.getDirectory()
        upload.upload(filepath)

    elif userOption in ["reset", "-r"]:
        filepath = navigation.getDirectory()
        reset.reset(filepath)

    elif userOption in ["help", "-h"]:
        print(strings.HELP_TEXT)

    else:
        print(strings.HELP_TEXT)


if __name__ == '__main__':
    main()



import os
import tkinter as tk
from tkinter import filedialog

from src import strings


def getDirectory():
    filepath = ""
    tk.Tk().withdraw()
    chosenFolder = filedialog.askdirectory(
        initialdir=filepath, title=strings.SELECT_FOLDER_TEXT)
    if chosenFolder not in ["", " ", "/"] and hasPyFiles(chosenFolder):
        filepath = chosenFolder
        return filepath
    else:
        quit()


def getPathFromBase(base, target):
    baseArray = base.split("/")
    targetArray = target.split("/")

    if len(baseArray) == len(targetArray):
        return strings.DEFAULT_MAIN_FUNCTION_PATH

    for index in range(len(baseArray)):
        if baseArray[index] != targetArray[index]:
            return strings.DEFAULT_MAIN_FUNCTION_PATH

    targetArray[-1] = targetArray[-1].split(".")[0]  # removing file extension
    return ".".join(targetArray[index + 1:])


def hasPyFiles(filepath):
    for _, _, files in os.walk(filepath):
        for file in files:
            if file.endswith(".py"):
                return True
    return False


def hasMainFunctionInSrc(filepath):
    for item in os.listdir(filepath):
        newPath = filepath + "/" + item
        if os.path.isdir(newPath) and os.path.basename(newPath) == "src":
            srcContents = os.listdir(newPath)
            for filename in srcContents:
                if filename == "__main__.py":
                    finalPath = newPath + "/" + filename
                    return mainFunctionIn(finalPath)
    return False


def mainFunctionIn(filepath):
    with open(filepath, "r") as file:
        for line in file:
            if line.startswith("def main("):
                return True
    return False



import datetime
import os
import subprocess

from src import strings
from src.ui.DialogWindow import DialogWindow


def generate(filepath, projectName, authorsArray, emailsArray, correspondingEmail, githubUsername,
             shortDescription, classifier, mainFunctionPath):
    checkForUnitTests(filepath)
    generateRequirementsFile(filepath)
    generateGitIgnoreFile(filepath)
    generateLicenseFile(filepath, authorsArray)
    generateReadmeFile(filepath, projectName, shortDescription, authorsArray, emailsArray,
                       githubUsername)
    generateInitFiles(filepath)
    generateSetupFile(filepath, authorsArray, correspondingEmail, githubUsername, projectName,
                      shortDescription, classifier, mainFunctionPath)


def checkForUnitTests(filepath):
    if "tests" not in os.listdir(filepath):
        dialogWindow = DialogWindow("No unit tests found", strings.UNIT_TEST_CREATION_PROMPT_TEXT,
                                    positiveButtonText="Create", negativeButtonText="Don't",
                                    positiveButtonAction=lambda: createUnitTestTemplates(
                                        filepath), negativeButtonColor="red")


def createUnitTestTemplates(filepath):
    pythonFileFilepaths = []
    appendPythonFileFilepathsToList(filepath, pythonFileFilepaths)
    newPaths = createNewPaths(filepath, pythonFileFilepaths)
    importStrings = createImportStrings(filepath, pythonFileFilepaths)
    createTestFiles(filepath, newPaths, importStrings)


def appendPythonFileFilepathsToList(filepath, pythonFileFilepaths):
    contents = os.listdir(filepath)
    for item in contents:
        if item.startswith(".") or item.startswith("__") or item == "setup.py":
            continue

        extension = item.split(".")[-1]
        if extension == "py":
            pythonFileFilepaths.append(filepath + "/" + item)

        itemPath = filepath + "/" + item
        if os.path.isdir(itemPath):
            appendPythonFileFilepathsToList(itemPath, pythonFileFilepaths)


def createImportStrings(filepath, pythonFileFilepaths):
    startingIndex = len(filepath.split("/"))
    importStrings = []
    for path in pythonFileFilepaths:
        pathWithoutExtension = path.split(".")[0]
        importStringPath = ".".join(
            pathWithoutExtension.split("/")[startingIndex:])
        filename = importStringPath.split(".")[-1]
        importString = f"import {importStringPath} as {filename}"
        importStrings.append(importString)
    return importStrings


def createNewPaths(filepath, pythonFileFilepaths):
    startingIndex = len(filepath.split("/"))
    newPaths = []
    for path in pythonFileFilepaths:
        pathArray = path.split("/")
        pathArray[startingIndex] = "tests"
        newPaths.append("/".join(pathArray))
    return newPaths


def createTestFiles(filepath, newPaths, importStrings):
    startingIndex = len(filepath.split("/"))
    for path, importString in zip(newPaths, importStrings):
        directory = "/".join(path.split("/")[:-1])
        contents = strings.TEST_FILE_CONTENTS.format(
            importString=importString,
            moduleName=path.split("/")[-1].split(".")[0].capitalize()
        )

        subprocess.call(["mkdir", "-p", directory])

        filename = path.split("/")[-1]
        newPathName = "".join([directory, "/", "test_", filename])
        with open(newPathName, "w") as file:
            file.write(contents)


def generateRequirementsFile(filepath):
    checkPipreqsInstallation()
    subprocess.run(["pipreqs", filepath], capture_output=True)


def checkPipreqsInstallation():
    try:
        import pipreqs
    except:
        commandArray = ["pip", "install", "pipreqs"]
        commandArray2 = ["pip3", "install", "pipreqs"]
        try:
            subprocess.call(commandArray)
        except:
            subprocess.call(commandArray2)
        print()


def generateGitIgnoreFile(filepath):
    output = ".DS_Store\n__pycache__/\n*.py[cod]"
    with open(filepath + "/.gitignore", "w") as file:
        file.write(output)


def generateLicenseFile(filepath, authorsArray):
    currentYear = datetime.datetime.now().year
    if len(authorsArray):
        copyrightLine = f"\nCopyright (c) {currentYear} {getAuthorNamesInline(authorsArray)}\n"
    else:
        copyrightLine = ""

    output = strings.LICENSE_FILE_CONTENTS.format(copyrightLine=copyrightLine)

    with open(filepath + "/LICENSE", "w") as file:
        file.write(output)


def getAuthorNamesInline(authorsArray):
    if not len(authorsArray):
        return ""

    elif len(authorsArray) == 1:
        return authorsArray[0]

    elif len(authorsArray) == 2:
        return f"{authorsArray[0]} and {authorsArray[1]}"

    else:
        allButLast = authorsArray[:-1]
        last = authorsArray[-1]
        return ", ".join(allButLast) + ", and " + last


def generateReadmeFile(filepath, projectName, blurb, authorsArray, emailsArray, githubUsername):
    if len(authorsArray):
        namesAndEmails = getNamesAndEmailsVerticallySpaced(
            authorsArray, emailsArray)
        authorSection = strings.AUTHOR_SECTION_CONTENTS.format(
            namesAndEmails=namesAndEmails)
    else:
        authorSection = ""

    if githubUsername == "":
        licenseDetails = ""
        bugReports = ""
    else:
        licenseDetails = strings.LICENSE_DETAILS_CONTENTS.format(
            githubUsername=githubUsername,
            projectName=projectName
        )
        bugReports = strings.BUG_REPORTS_CONTENTS.format(
            githubUsername=githubUsername,
            projectName=projectName
        )

    output = strings.README_CONTENTS.format(
        projectName=projectName,
        blurbLine="" if blurb == "" else f"> {blurb}\n",
        authorSection=authorSection,
        version=strings.DEFAULT_INITIAL_VERSION,
        licenseDetails=licenseDetails,
        bugReports=bugReports
    )

    with open(filepath + "/README.md", "w") as file:
        file.write(output)


def getNamesAndEmailsVerticallySpaced(authorsArray, emailsArray):
    namesAndEmails = []
    for name, email in zip(authorsArray, emailsArray):
        if email == "":
            namesAndEmails.append(name)
        else:
            namesAndEmails.append(f"{name} ({email})")
    return "\n\n".join(namesAndEmails)


def generateInitFiles(filepath):
    searchForPyFiles(filepath, includeThisDirectory=False)


def searchForPyFiles(filepath, includeThisDirectory=True):
    pyFileFound = False
    contents = os.listdir(filepath)
    for item in contents:
        if item.startswith("."):
            continue
        extension = item.split(".")[-1]
        if includeThisDirectory and extension == "py" and not pyFileFound:
            createInitFileAt(filepath)
            pyFileFound = True

        itemPath = filepath + "/" + item
        if os.path.isdir(itemPath):
            searchForPyFiles(itemPath)


def createInitFileAt(filepath):
    with open(filepath + "/__init__.py", "w") as openFile:
        openFile.write("")


def generateSetupFile(filepath, authorsArray, correspondingEmail, githubUsername, projectName,
                      shortDescription, classifier, mainFunctionPath):
    githubInfo = getGithubInfo(githubUsername, projectName)

    output = strings.SETUP_FILE_CONTENTS.format(
        projectName=projectName,
        version=strings.DEFAULT_INITIAL_VERSION,
        descriptionLine=getDescriptionLine(shortDescription),
        githubUrlLine=githubInfo[0],
        authorNameLine=getAuthorsNameLine(authorsArray),
        authorEmailLine=getAuthorEmailLine(correspondingEmail),
        classifiers=getClassifiers(classifier),
        pyModules=getPyModules(filepath),
        githubProjectUrlsLine=githubInfo[1],
        mainFunctionPath=mainFunctionPath
    )

    with open(filepath + "/setup.py", "w") as file:
        file.write(output)


def getPyModules(filepath):
    pyModules = []
    searchForModules(filepath, filepath, pyModules)
    return pyModules


def searchForModules(absoluteFilepath, filepath, pyModules):
    contents = os.listdir(filepath)
    for item in contents:
        if item.startswith(".") or item.endswith("egg-info") or item in ["build", "dist"]:
            continue
        elif item == "__init__.py":
            pyModules.append(getFormattedPath(absoluteFilepath, filepath))

        itemPath = filepath + "/" + item
        if os.path.isdir(itemPath):
            searchForModules(absoluteFilepath, itemPath, pyModules)


def getFormattedPath(absoluteFilepath, filepath):
    topLevelFolderName = absoluteFilepath.split("/")[-1]
    filepathArray = filepath.split("/")
    index = 0
    folderName = filepathArray[index]
    while folderName != topLevelFolderName:
        folderName = filepathArray[index]
        index += 1
    return ".".join(filepathArray[index:])


def getAuthorsNameLine(authorsArray):
    if not len(authorsArray):
        return ""
    else:
        return f"\n    author='{getAuthorNamesInline(authorsArray)}',"


def getAuthorEmailLine(correspondingEmail):
    return f"\n    author_email='{correspondingEmail}'," if correspondingEmail != "" else ""


def getDescriptionLine(description):
    return f"\n    description='{description}'," if description != "" else ""


def getGithubInfo(username, projectName):
    if username == "":
        return ["", ""]
    else:
        urlLine = strings.GITHUB_URL_LINE_CONTENT.format(
            username=username, projectName=projectName)
        projectUrlsLine = strings.GITHUB_PROJECT_URLS_LINE_CONTENT.format(
            username=username,
            projectName=projectName
        )
        return [urlLine, projectUrlsLine]


def getClassifiers(classifier):
    if classifier in ["N/A", "Select..."]:
        return ""
    else:
        return strings.CLASSIFIERS_CONTENT.format(classifier=classifier)



import unittest

from src import strings
import src.ui.navigation as navigation

# This unit test uses Python's built-in unit testing framework
# See https://docs.python.org/3/library/unittest.html for more information


class NavigationTest(unittest.TestCase):
    def test_getPathFromBase(self):
        self.assertEqual(
            navigation.getPathFromBase(
                base="path/to/base", target="path/to/base"),
            strings.DEFAULT_MAIN_FUNCTION_PATH
        )
        self.assertEqual(
            navigation.getPathFromBase(
                base="path/to/base", target="not/path/to/base"),
            strings.DEFAULT_MAIN_FUNCTION_PATH
        )
        self.assertEqual(
            navigation.getPathFromBase(
                base="path/to/base", target="path/not/to/base/to/target.py"),
            strings.DEFAULT_MAIN_FUNCTION_PATH
        )
        self.assertEqual(
            navigation.getPathFromBase(
                base="path/to/base", target="path/to/base/to/target.py"),
            "to.target"
        )
        self.assertEqual(
            navigation.getPathFromBase(
                base="path/to/base", target="path/to/base/to/__target__.py"),
            "to.__target__"
        )


if __name__ == "__main__":
    unittest.main()



ARGS_DESCRIPTION_TEXT = "Enter command 'generate', 'upload', 'reset', or 'help'"

HELP_TEXT = """
PyPI Maker
----------

To run PyPI Maker, enter 'pypimaker' followed by a valid command.
Some examples:

   pypimaker generate
   pypimaker upload
   pypimaker -r

The possible commands (or their aliases) are the following:

   generate  (-g)
   upload    (-u)
   reset     (-r)
   help      (-h)

See the documentation on GitHub for more information:
https://github.com/bdavis222/pypimaker
"""

DEFAULT_INITIAL_VERSION = "0.1.0"

PROJECT_DESCRIPTION_TEXT = 'One-sentence Project Description (e.g., "Software designed for..."):'
SELECT_FOLDER_TEXT = "Select the top-level folder containing your Python project"
PROJECT_NAME_DESCRIPTION_TEXT = '(Your package will be installed with the "pip install \
project_name" command using the project name given above.)'

MISMATCHED_SELECTION_TEXT = "Mismatched folder name and project name.\nAre you sure you selected correctly?"
NO_PYTHON_FILES_IN_PROJECT_FOLDER_TEXT = "No Python files found in folder.\nSelect a different folder."
PROJECT_FOLDER_NOT_CHOSEN_TEXT = "You must select your project \nusing the Browse button."
PROJECT_NAME_NOT_CHOSEN_TEXT = "Project Name is a required field.\nUse folder name as project name?"

NO_MAIN_FUNCTION_TITLE = "Main function not found"
MUST_SELECT_MAIN_FILE = 'No "main" function found.\nChoose a file containing\nthe main entry point function?'
NO_MAIN_FOUND_AFTER_SELECTION_TEXT = 'The selected file does not contain\na function named "main" for your project.'

MISSING_AUTHOR_NAMES_TEXT = "Some emails have missing author names.\nPlease fix and resubmit."
MISSING_FIRST_NAME_TEXT = "Corresponding author name not given.\nSubmit with no authors?"
MISSING_FIRST_AUTHOR_EMAIL_TEXT = "Email given for other(s), but not first author.\n\
Use next email in list for correspondence?"

RESET_SELECTION_TEXT = "This will remove generated files.\nAre you sure you want to reset?"

UNIT_TEST_CREATION_PROMPT_TEXT = '''Create template unit test files?
These will be placed in a "tests" folder
within your main project folder.'''

TEST_FILE_CONTENTS = '''import unittest
{importString}

# This unit test uses Python's built-in unit testing framework
# See https://docs.python.org/3/library/unittest.html for more information

class {moduleName}Test(unittest.TestCase):
    def test_condition_doesThis_whenThis(self):
        pass

if __name__ == "__main__":
    unittest.main()
'''

LICENSE_FILE_CONTENTS = '''MIT License
{copyrightLine}
Permission is hereby granted, free of charge, to any person obtaining a copy \
of this software and associated documentation files (the "Software"), to deal \
in the Software without restriction, including without limitation the rights \
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell \
copies of the Software, and to permit persons to whom the Software is \
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all \
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR \
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, \
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE \
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER \
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, \
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE \
SOFTWARE.'''

README_CONTENTS = """# {projectName}
{blurbLine}
This Python package was created and uploaded to PyPI using [PyPI Maker](https://github.com/bdavis222/pypimaker). \
This is a template README generated by PyPI Maker.

[![Donate](https://img.shields.io/badge/Donate-PayPal-green.svg)](PAYPAL_DONATE_LINK_GOES_HERE)

OPTIONAL: ENTER A 2-3 SENTENCE DESCRIPTION ABOUT THE SOFTWARE HERE

## Getting Started

### Dependencies

[Python 3](https://www.python.org/downloads/) must be installed before {projectName} can be installed.

### Installation

To install {projectName}, open a terminal window and run the following command:

```
pip install {projectName}
```

*(Note that the* `pip3` *command may be required instead of* `pip` *for some Python installations.)*

### Running the Software

To launch the main program, run the following command:

```
{projectName}
```

## SOME_OTHER_HEADER

IMAGES INCLUDED BY LINKING TO THEIR LOCATION IN THE PROJECT, E.G.:
![](https://github.com/bdavis222/dotscanner/blob/main/images/3.png)
{authorSection}
## Release History

* {version}
     * Initial Release

## License

This project is licensed under the MIT License. {licenseDetails}

## Development

To contribute, download or clone the project. From the top level of the project's folder structure, \
one can use the following command to run a local version of the software (e.g., for UI testing):

```
python -m src
```

*(Note that the* `python3` *command may be required instead of* `python` *for some Python installations.)*

### Testing

Unit tests for this software were written for use with [Python's built-in unittest \
framework](https://docs.python.org/3/library/unittest.html), and are stored in the `tests` folder. \
To run tests, download the project, navigate to the top level of the project's folder structure and \
run the following command:

```
python -m unittest
```
{bugReports}
"""

AUTHOR_SECTION_CONTENTS = "\n## Authors\n\n{namesAndEmails}\n"

LICENSE_DETAILS_CONTENTS = "See the [LICENSE](https://github.com/{githubUsername}/{projectName}/blob/main/LICENSE) file for details."

BUG_REPORTS_CONTENTS = """
### Bug Reports and Feature Requests\n\

To report a bug, visit the [issues page](https://github.com/{githubUsername}/{projectName}/issues). \
New feature requests are also welcome!"""

SETUP_FILE_CONTENTS = """from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
    name='{projectName}', # Required
    version='{version}', # Required {descriptionLine}
    long_description=long_description,
    long_description_content_type='text/markdown',\
{githubUrlLine}{authorNameLine}{authorEmailLine}{classifiers}
    packages=find_packages(), # Required
    py_modules={pyModules}, # Generated
    python_requires='>=3.7, <4', {githubProjectUrlsLine}
    entry_points={{
        'console_scripts': [
            '{projectName} = {mainFunctionPath}:main'
        ]
    }}
)
"""

DEFAULT_MAIN_FUNCTION_PATH = "src.__main__"

GITHUB_URL_LINE_CONTENT = "\n    url='https://github.com/{username}/{projectName}',"

GITHUB_PROJECT_URLS_LINE_CONTENT = """
    project_urls={{
        'Bug Reports': 'https://github.com/{username}/{projectName}/issues',
        # 'Funding': 'PAYPAL_DONATE_LINK_GOES_HERE',
        'Source': 'https://github.com/{username}/',
    }},"""

CLASSIFIERS_CONTENT = """
    classifiers=[ # Defined at https://pypi.org/classifiers/
        'Intended Audience :: {classifier}',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Operating System :: Unix',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows'
    ],"""

INTENDED_AUDIENCES_LIST = [
    "N/A",
    "Customer Service",
    "Developers",
    "Education",
    "End Users/Desktop",
    "Financial and Insurance Industry",
    "Healthcare Industry",
    "Information Technology",
    "Legal Industry",
    "Manufacturing",
    "Other Audience",
    "Religion",
    "Science/Research",
    "System Administrators",
    "Telecommunications Industry"
]



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
    name='pypimaker',  # Required
    version='1.0.2', # Required 
    description='Software designed for simplifying PyPI Python package setups',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/bdavis222/pypimaker',
    author='Brian Davis',
        classifiers=[  # Defined at https://pypi.org/classifiers/
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 3',
            'Operating System :: Unix',
            'Operating System :: MacOS :: MacOS X',
            'Operating System :: Microsoft :: Windows'
        ],
    packages=find_packages(),  # Required
    py_modules=['src', 'src.ui', 'tests'],  # Generated
    python_requires='>=3.7, <4',
    project_urls={
        'Bug Reports': 'https://github.com/bdavis222/pypimaker/issues',
        'Funding': 'https://www.paypal.com/donate/?business=UA5NL9MJSFMVY',
        'Source': 'https://github.com/bdavis222/',
    },
    entry_points={
        'console_scripts': [
            'pypimaker = src.__main__:main'
        ]
    }
)



import tkinter as tk


class DialogWindow:
    def __init__(self, title, message, positiveButtonText, negativeButtonText,
                 positiveButtonAction=None, negativeButtonAction=None, windowWidth=300,
                 windowHeight=140, positiveButtonColor="blue", negativeButtonColor="black",
                 mainWindow=False):
        self.window = tk.Tk()
        self.window.title(title)
        centerX = (self.window.winfo_screenwidth() - windowWidth) // 2
        centerY = (self.window.winfo_screenheight() - windowHeight) // 2
        self.window.geometry(
            f"{windowWidth}x{windowHeight}+{centerX}+{centerY}")

        self.positiveButtonAction = positiveButtonAction
        self.negativeButtonAction = negativeButtonAction

        spacer = tk.Label(self.window, text=" ")
        messageText = tk.Label(self.window, text=message)
        spacer2 = tk.Label(self.window, text=" ")

        buttonsFrame = tk.Frame(self.window)

        if positiveButtonAction is None:
            positiveButton = tk.Button(
                self.window, text=positiveButtonText, fg=positiveButtonColor,
                command=self.closeWindow)
            self.window.bind("<Return>", self.closeWindowWithReturnKey)
        else:
            positiveButton = tk.Button(self.window, text=positiveButtonText,
                                       fg=positiveButtonColor, command=self.performPositiveAction)
            self.window.bind(
                "<Return>", self.performPositiveActionWithReturnKey)

        if negativeButtonAction is None:
            negativeButton = tk.Button(
                self.window, text=negativeButtonText, fg=negativeButtonColor,
                command=self.closeWindow)
            self.window.bind("<Escape>", self.closeWindowWithEscapeKey)
        else:
            negativeButton = tk.Button(self.window, text=negativeButtonText,
                                       fg=negativeButtonColor, command=self.performNegativeAction)
            if negativeButtonAction == exit:
                self.window.bind(
                    "<Escape>", self.performNegativeActionWithEscapeKey)

        negativeButton.pack(in_=buttonsFrame, side=tk.LEFT)
        positiveButton.pack(in_=buttonsFrame, side=tk.LEFT)

        spacer.pack()
        messageText.pack()
        spacer2.pack()
        buttonsFrame.pack()

        if mainWindow:
            self.window.protocol("WM_DELETE_WINDOW", quit)

        self.window.mainloop()

    def closeWindow(self):
        self.window.destroy()
        self.window.quit()

    def closeWindowWithEscapeKey(self, _):
        self.closeWindow()

    def closeWindowWithReturnKey(self, _):
        self.closeWindow()

    def performPositiveAction(self):
        self.closeWindow()
        self.positiveButtonAction()

    def performPositiveActionWithReturnKey(self, _):
        self.closeWindow()
        self.positiveButtonAction()

    def performNegativeAction(self):
        self.closeWindow()
        self.negativeButtonAction()

    def performNegativeActionWithEscapeKey(self, _):
        self.closeWindow()
        self.negativeButtonAction()


class ActionDialogWindow(DialogWindow):
    def __init__(self, title, message, positiveAction=None, negativeAction=None,
                 negativeColor="red", mainWindow=False):
        super().__init__(title, message, positiveButtonText="Confirm", negativeButtonText="Cancel",
                         positiveButtonAction=positiveAction, negativeButtonAction=negativeAction,
                         negativeButtonColor=negativeColor, mainWindow=mainWindow)


class InfoDialogWindow(DialogWindow):
    def __init__(self, title, message, negativeColor="black"):
        super().__init__(title, message, positiveButtonText="Ok", negativeButtonText="Go back",
                         negativeButtonColor=negativeColor)



import os
import subprocess
import time

from src import strings


def upload(filepath):
    checkForRequiredFiles(filepath)

    checkWheelInstallation()
    checkTwineInstallation()

    versionNumber = getValidVersionNumberForSetupFile(filepath)
    setNewSetupFileVersion(filepath, versionNumber)

    createDistributionFiles(filepath)
    uploadDistributionFiles(filepath)
    removeDistributionFiles(filepath)


def checkForRequiredFiles(filepath):
    requiredFiles = ["LICENSE", "README.md", "requirements.txt", "setup.py"]
    topLevelDirectoryFiles = set(os.listdir(filepath))
    for file in requiredFiles:
        if file not in topLevelDirectoryFiles:
            print(f"{file} not yet created. Run generatefiles.py first!")
            quit()

    if "build" in topLevelDirectoryFiles or "dist" in topLevelDirectoryFiles:
        print("Top-level folders cannot be named 'build' or 'dist'. Please change folder names.")
        quit()


def checkWheelInstallation():
    try:
        import wheel
    except:
        commandArray = ["pip", "install", "wheel"]
        commandArray2 = ["pip3", "install", "wheel"]
        try:
            subprocess.call(commandArray)
        except:
            subprocess.call(commandArray2)
        print()


def checkTwineInstallation():
    try:
        import twine
    except:
        commandArray = ["pip", "install", "twine"]
        commandArray2 = ["pip3", "install", "twine"]
        try:
            subprocess.call(commandArray)
        except:
            subprocess.call(commandArray2)
        print()


def getValidVersionNumberForSetupFile(filepath):
    currentSetupFileVersion = getSetupFileVersion(filepath)
    currentPackageVersion = getCurrentPackageVersion(
        getPackageNameFromSetupFile(filepath))
    if not versionNumberGreater(currentSetupFileVersion, currentPackageVersion):
        currentSetupFileVersion = getNewSetupFileVersion(currentPackageVersion)
    return currentSetupFileVersion


def getSetupFileVersion(filepath):
    with open(filepath + "/setup.py", "r") as file:
        data = file.readlines()

    for rawLine in data:
        line = rawLine.split()
        if not len(line):
            continue

        if line[0].startswith("version"):
            fullLine = "".join(line)
            currentVersion = fullLine.split("'")[1]
            return currentVersion

    return strings.DEFAULT_INITIAL_VERSION


def getCurrentPackageVersion(packageName):
    commandArray = ['pip', 'install', f'{packageName}==nonVersionString']
    result = subprocess.run(commandArray, capture_output=True)
    errorMessage = str(result.stderr)
    if "versions:" not in errorMessage:
        return "0.0.0"
    versions = errorMessage.split("versions:")[-1].split(")")[0].split()
    versions = [version[:-1]
                if version.endswith(",") else version for version in versions]
    return "0.0.0" if versions == ["none"] else getLargestVersionNumber(versions)


def getPackageNameFromSetupFile(filepath):
    with open(filepath + "/setup.py", "r") as file:
        data = file.readlines()

    for rawLine in data:
        line = rawLine.split()
        if not len(line):
            continue

        if line[0].startswith("name"):
            fullLine = "".join(line)
            packageName = fullLine.split("'")[1]
            return packageName

    raise Exception("No package name found in setup.py")


def versionNumberGreater(largerVersion, smallerVersion):
    largerVersionArray = [int(version) for version in largerVersion.split(".")]
    smallerVersionArray = [int(version)
                           for version in smallerVersion.split(".")]
    largerVersionFirst, largerVersionSecond, largerVersionThird = largerVersionArray
    smallerVersionFirst, smallerVersionSecond, smallerVersionThird = smallerVersionArray

    if largerVersionFirst > smallerVersionFirst:
        return True
    elif largerVersionFirst < smallerVersionFirst:
        return False

    if largerVersionSecond > smallerVersionSecond:
        return True
    elif largerVersionSecond < smallerVersionSecond:
        return False

    return largerVersionThird > smallerVersionThird


def getNewSetupFileVersion(currentPackageVersion):
    newVersion = ""
    while newVersion == "":
        newVersion = input(f"The current version number ({currentPackageVersion}) must be \
incremented. Enter new version number: ")

        if not isValidVersionNumber(newVersion):
            newVersion = ""
            continue

        if not versionNumberGreater(newVersion, currentPackageVersion):
            newVersion = ""
            continue

    print()
    return newVersion


def getLargestVersionNumber(versions):
    maxVersion = "0.0.0"
    for version in versions:
        if versionNumberGreater(version, maxVersion):
            maxVersion = version
    return maxVersion


def isValidVersionNumber(versionNumber):
    numberArray = versionNumber.split(".")
    if len(numberArray) != 3:
        return False
    try:
        int(numberArray[0])
        int(numberArray[1])
        int(numberArray[2])
        return True
    except:
        return False


def setNewSetupFileVersion(filepath, newVersionNumber):
    versionNumberFound = False
    with open(filepath + "/setup.py", "r") as file:
        data = file.readlines()

    for index, rawLine in enumerate(data):
        line = rawLine.split()
        if not len(line):
            continue

        if line[0].startswith("version"):
            versionIndex = index
            versionNumberFound = True
            break

    if not versionNumberFound:
        raise Exception("No version number found in setup.py")

    data[versionIndex] = f"    version='{newVersionNumber}', # Required \n"

    with open(filepath + "/setup.py", "w") as file:
        file.writelines(data)


def createDistributionFiles(filepath):
    setupFilePath = filepath + "/setup.py"

    commandArray = ["python", "setup.py", "sdist", "bdist_wheel"]
    commandArray2 = ["python3", "setup.py", "sdist", "bdist_wheel"]
    try:
        subprocess.call(commandArray, cwd=filepath)
    except:
        subprocess.call(commandArray2, cwd=filepath)

    print()

    count = 0
    while "dist" not in os.listdir(filepath):
        time.sleep(0.1)
        count += 1
        if count > 50:
            raise Exception("'dist' directory not created successfully")


def uploadDistributionFiles(filepath):
    distPath = filepath + "/dist"

    count = 0
    while len(os.listdir(distPath)) < 2:
        time.sleep(0.1)
        count += 1
        if count > 50:
            raise Exception(
                "Files in 'dist' directory not created successfully")

    commandArray = ["twine", "upload", distPath + "/*"]
    subprocess.call(commandArray)


def removeDistributionFiles(filepath):
    foldersToRemove = ["build", "dist"]
    for item in os.listdir(filepath):
        if item.endswith("egg-info"):
            foldersToRemove.append(item)

    for folder in foldersToRemove:
        commandArray = ["rm", "-r", filepath + "/" + folder]
        subprocess.run(commandArray, capture_output=True)



import os
import subprocess

from src import strings
from src.ui.DialogWindow import ActionDialogWindow


def reset(topLevelDirectory):
    ActionDialogWindow("Are you sure?", strings.RESET_SELECTION_TEXT,
                       negativeAction=exit, mainWindow=True)

    filesToRemove = ["LICENSE", "README.md",
                     "requirements.txt", "setup.py", ".gitignore"]
    foldersToRemove = ["build", "dist"]
    for item in os.listdir(topLevelDirectory):
        if item.endswith("egg-info"):
            foldersToRemove.append(item)

    for file in filesToRemove:
        commandArray = ["rm", topLevelDirectory + "/" + file]
        subprocess.run(commandArray, capture_output=True)

    for folder in foldersToRemove:
        commandArray = ["rm", "-r", topLevelDirectory + "/" + folder]
        subprocess.run(commandArray, capture_output=True)
